<?php

/**
 * xt:Commerce plugin class.
 *
 * Extends the payment
 *
 * @author S-Public Services
 * @link https://s-publicservices.de
 * @package GiroCheckout
 * @copyright Copyright (c) 2014, S-Public Services
 */
defined('_VALID_CALL') or die('Direct Access is not allowed.');
require_once(dirname(__FILE__) . '/api/GiroCheckout_SDK.php');

GiroCheckout_SDK_Config::getInstance()->setConfig('DEBUG_LOG_PATH', _SRV_WEBROOT . '/xtLogs/girocheckout');

//GiroCheckout_SDK_Config::getInstance()->setConfig('DEBUG_MODE',1);

class xt_girocheckout extends payment
{

  public $data = array();
  public $version = '6.0.1';
  public $external = true;
  public $iframe = false;
  public $post_form = false;
  public $subpayments = true;
  public $allowed_subpayments;
  public $lang = 'de';
  public $sourceID = 'c3ba6c99e342084767a311a8460db065';
  public $aTexts = array();
  public $sysversion4 = 'false';

  const CART_TYPE_PHYSICAL = "PHYSICAL";
  const CART_TYPE_DIGITAL = "DIGITAL";
  const CART_TYPE_MIXED = "MIXED";

  function __construct() {
    global $page, $xtLink, $language;

    $this->lang = 'de';
    if (isset($language)) {
      if ($language->code === 'en') {
        $this->lang = $language->code;
      }
    }

    $this->allowed_subpayments = array(
      'gc_sofortuw',
      'gc_giropay',
      'gc_directdebit',
      'gc_creditcard',
      'gc_ideal',
      'gc_eps',
      'gc_paydirekt',
      'gc_bluecode',
      'gc_maestro'
    );

    $this->loadText();

    // Circumvent the selected payment sub name.
    foreach ($this->allowed_subpayments as $subModule) {
      $subDef = 'TEXT_PAYMENT_' . strtoupper($subModule);
      $subDefName = $this->aTexts[$this->lang][$subModule]['title'];
      if (!defined($subDef)) {
        define('TEXT_PAYMENT_' . strtoupper($subModule), $subDefName);
      }
    }

    $this->getConfFieldsBySubModules();
    $this->getSubModulesInfo();
    $this->handlePageAction($page->page_name, $page->page_action);
    uasort($this->data, array($this, "sortMethods"));

    $this->data["aPayments"] = $this->data;

  }

  /**
   * Check if the payment method is allowed.
   *
   * @author S-Public Services
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, S-Public Services
   * @param string $subModule The payment method
   * @return bool
   */
  function subModuleIsAllowed($subModule = '') {
    return in_array($subModule, $this->allowed_subpayments);
  }

  /**
   * Check if the payment method is activated.
   *
   * @author S-Public Services
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, S-Public Services
   * @param string $subModule The payment method
   * @return bool
   */
  function subModuleIsActivated($subModule = '') {
    $subModulePref = strtoupper(get_class($this) . '_' . $subModule . '_');
    if (defined($subModulePref . 'ACTIVATED')) {
      if (constant($subModulePref . 'ACTIVATED') == 'true') {
        return true;
      }
    }

    return false;
  }

  /**
   * Check the payment method state.
   *
   * @author S-Public Services
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, S-Public Services
   * @param string $subModule The payment method
   * @return bool
   */
  function checkSubModule($subModule = '') {
    global $xtLink;

    if ($this->subModuleIsAllowed($subModule)) {
      if ($this->subModuleIsActivated($subModule)) {
        return true;
      }
    }

    return false;
  }

  /**
   * Get the payments method credentials.
   *
   * @author S-Public Services
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, S-Public Services
   * @return none
   */
  function getConfFieldsBySubModules() {
    global $currency;

    foreach ($this->allowed_subpayments as $subModule) {
      if ($this->checkSubModule($subModule)) {
        $subModulePref = strtoupper(get_class($this) . '_' . $subModule . '_');
        $params = array();
        $params['enable'] = (constant($subModulePref . 'ACTIVATED') == 'true' ? true : false);
        $params['merchantId'] = constant($subModulePref . 'MID');
        $params['projectId'] = constant($subModulePref . 'PID');
        $params['secure'] = constant($subModulePref . 'SECURE');
        $params['purpose'] = constant($subModulePref . 'PURPOSE');
        $params['order'] = constant($subModulePref . 'ORDER');

        switch ($subModule) {
          case 'gc_giropay':
          case 'gc_ideal':
          case 'gc_eps':
          case 'gc_sofortuw':
          case 'gc_directdebit':
          case 'gc_paydirekt':
          case 'gc_bluecode':
          case 'gc_maestro':
            if ($params['enable'] && $currency->code != "EUR") {
              $params['enable'] = false;
            }
            break;
          case 'gc_creditcard':
            $params['visamaster'] = constant($subModulePref . 'VISAMASTER');
            $params['amex'] = constant($subModulePref . 'AMEX');
            $params['jcb'] = constant($subModulePref . 'JCB');
            break;
        }
        $this->data[$subModule] = $params;
      }
    }
  }

  /**
   * Load the payments method information.
   *
   * @author S-Public Services
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, S-Public Services
   * @return none
   */
  function getSubModulesInfo() {
    // If version is less 5 
    if (version_compare(5, _SYSTEM_VERSION) == 1) {
      $this->sysversion4 = 'true';
    }

    $this->data['xt_girochekout']['sysversion4'] = $this->sysversion4;
    $this->data['xt_girochekout']['payment_icon'] = _SYSTEM_BASE_URL . _SRV_WEB . _SRV_WEB_PLUGINS . 'xt_girocheckout/images/girocheckout_logo.png';
    $this->data['xt_girochekout']['titleplugin'] = 'Bezahlen mit:';
    foreach ($this->allowed_subpayments as $subModule) {

      if ($this->checkSubModule($subModule)) {
        $title = $this->aTexts[$this->lang][$subModule]['title'];
        $title[0] = strtoupper($title);
        $this->data[$subModule]['title'] = htmlspecialchars($title);
        $this->data[$subModule]['description'] = htmlspecialchars($this->aTexts[$this->lang][$subModule]['description']);

        if ($subModule == 'gc_creditcard') {

          $strLogoName = $this->getExtendedLogo();

          if (strlen($strLogoName) > 0) {
            $strLogoImageCredit = $strLogoName;
          }
          else {
            $strLogoImageCredit = 'gc_creditcard.jpg';
          }

          $this->data[$subModule]['imageUrl'] = _SYSTEM_BASE_URL . _SRV_WEB . _SRV_WEB_PLUGINS . 'xt_girocheckout/images/' . $strLogoImageCredit;
        }
        else {
          if ($subModule == 'gc_directdebit' || $subModule == 'gc_paydirekt' || $subModule == 'gc_sofortuw' ||
            $subModule == 'gc_ideal' || $subModule == 'gc_bluecode' || $subModule == 'gc_maestro'
          ) {
            $this->data[$subModule]['imageUrl'] = _SYSTEM_BASE_URL . _SRV_WEB . _SRV_WEB_PLUGINS . 'xt_girocheckout/images/' . $subModule . '.png';
          }
          else {
            $this->data[$subModule]['imageUrl'] = _SYSTEM_BASE_URL . _SRV_WEB . _SRV_WEB_PLUGINS . 'xt_girocheckout/images/' . $subModule . '.jpg';
          }
        }
      }
    }
  }


  function sortMethods($a, $b) {
    return ($a["order"] <= $b["order"]) ? -1 : 1;
  }

  /**
   * Check the payment name selegetSubModuleNamected.
   *
   * @author S-Public Services
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, S-Public Services
   * @return bool
   */
  function getSubModuleName() {
    global $_POST;

    if (isset($_POST['selected_payment'])) {
      $selectedPayment = $_POST['selected_payment'];
      if (strpos($selectedPayment, ':')) {
        $payments = explode(':', $selectedPayment);
        $subModule = $payments[1];
        if ($this->checkSubModule($subModule)) {
          return $subModule;
        }
      }
    }
    else {
      if (isset($_SESSION['selected_payment_sub'])) {
        return $_SESSION['selected_payment_sub'];
      }
      else {
        if (isset($processed_data['subpayment_code'])) {
          return $processed_data['subpayment_code'];
        }
      }
    }

    return false;
  }

  /**
   * Clear the plugin session.
   *
   * @author S-Public Services
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, S-Public Services
   * @return none
   */
  static function clearSession() {
    if (isset($_SESSION['xt_girocheckout'])) {
      unset($_SESSION['xt_girocheckout']);
    }
  }

  /**
   * Set the error message in session.
   *
   * @author S-Public Services
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, S-Public Services
   * @param string $msg The error message
   * @return none
   */
  static function setErrorMsg($msg = '') {
    $_SESSION['xt_girocheckout']['error'] = $msg;
  }

  /**
   * Handle the page action.
   *
   * @author S-Public Services
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, S-Public Services
   * @return none
   */
  function handlePageAction($page, $action) {
    global $_POST, $filter, $xtLink, $info, $order;

    if ($page == 'checkout') {
      //Sometimes the 'payment' stage can be 'confirmation', bug in Veyton?
      if ($action == 'payment' || $action == 'confirmation') {

      }

      if ($action == 'confirmation' && $_SESSION['selected_payment'] == get_class($this)) {

      }

      if ($action == 'shipping') {

      }
    }
  }

  /**
   * Build the payment information.
   *
   * @author S-Public Services
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, S-Public Services
   * @param array $data
   * @return bool
   */
  function build_payment_info($data) {
    return true;
  }

  /**
   * Process the payment transaction and redirect to the bank page.
   *
   * @author S-Public Services
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, S-Public Services
   * @param array $processed_data
   * @return string, the link page
   */
  function pspRedirect($processed_data = array()) {
    global $xtLink, $filter, $order, $db;

    // cancel
    $tmp_link = $xtLink->_link(array(
      'page' => 'checkout',
      'paction' => 'payment',
      'params' => $xtLink->_getParams() . '&error_payments=ERROR_PAYMENT',
      'conn' => 'SSL'
    ));

    if (($subModule = $this->getSubModuleName()) !== false) {

      if ($this->checkSubModule($subModule)) {
        $customers_id = $order->customer;
        $cart = $_SESSION['cart'];
        $amount = trim(round($cart->total['plain'], 2));
        $currency = isset($processed_data['currency_code']) ? $processed_data['currency_code'] : 'EUR';
        $transaction_id = (int)$processed_data['orders_id'];
        $purpose = $this->data[$subModule]['purpose'];
        $vwz = $this->getPurpose($purpose, $order);//'Best. ' . $transaction_id;
        $amountShipping = trim(round($cart->sub_content_total['plain'], 2));
        $amountOrder = $amount - $amountShipping;

        $urlNotify = str_replace('&amp;', '&', $xtLink->_link(array(
          'page' => 'callback',
          'paction' => 'xt_girocheckout',
          'params' => 'act=notify&pm=' . $subModule
        )));
        $urlRedirect = str_replace('&amp;', '&', $xtLink->_link(array(
          'page' => 'callback',
          'paction' => 'xt_girocheckout',
          'params' => 'act=redirect&pm=' . $subModule
        )));

        // Get the shipping amount
        foreach ($order->order_total_data as $oOrderShipping) {
          if ($oOrderShipping["orders_total_key"] == 'shipping') {
            $amountShipping = trim(round($oOrderShipping["orders_total_price"]['plain'], 2));
          }
        }

        $order_query = "SELECT orders_id, customers_id FROM " . TABLE_ORDERS . " WHERE orders_id = '" . (int)$transaction_id . "'";
        $rs = $db->Execute($order_query);

        if ($rs->RecordCount() == 1) {
          $orders_id = $rs->fields['orders_id'];
        }

        switch ($subModule) {
          case 'gc_giropay': //gc_giropay
            $merchantId = $this->data[$subModule]['merchantId'];
            $projectId = $this->data[$subModule]['projectId'];
            $secure = $this->data[$subModule]['secure'];
            $vwz = $this->getGiropayPurpose($purpose, $order);

            $iNumProdVirtual = 0;
            $iNumProdPhysical = 0;
            $strCartType = self::CART_TYPE_PHYSICAL;
            $oCart = new GiroCheckout_SDK_Request_Cart();

            try {
              foreach ($order->order_products as $oProduct) {
                $oCart->addItem($oProduct["products_name"], trim(round($oProduct["products_quantity"], 2)),
                  trim(round($oProduct["products_price"]["plain"], 2)) * 100, $oProduct["products_model"]);

                if ($oProduct["products_digital"]) {
                  $iNumProdVirtual++;
                }
                else {
                  $iNumProdPhysical++;
                }
              }

              // Set the shopping cart type
              if ($iNumProdVirtual > 0 && $iNumProdPhysical == 0) {
                $strCartType = self::CART_TYPE_DIGITAL;

              }
              elseif ($iNumProdVirtual > 0 && $iNumProdPhysical > 0) {
                $strCartType = self::CART_TYPE_MIXED;
              }

              $strVorname = $order->order_data["delivery_firstname"];
              $strNachname = $order->order_data["delivery_lastname"];
              $strFirma = $order->order_data["delivery_company"];
              $strAdditionalAdr = $order->order_data["comments"];
              $strStrasse = $order->order_data["delivery_street_address"];
              $strPLZ = $order->order_data["delivery_postcode"];
              $strOrt = $order->order_data["delivery_city"];
              $strLand = $order->order_data["delivery_country_code"];
              $strMail = $order->order_data["customers_email_address"];

              //Sends request to Girocheckout.
              $request = new GiroCheckout_SDK_Request('giropayTransaction');
              $request->setSecret($secure);
              $request->addParam('merchantId', $merchantId)
                ->addParam('projectId', $projectId)
                ->addParam('merchantTxId', $transaction_id)
                ->addParam('amount', $amount * 100)
                ->addParam('currency', $currency)
                ->addParam('purpose', $vwz)
                ->addParam('urlRedirect', $urlRedirect)
                ->addParam('urlNotify', $urlNotify)
                ->addParam('sourceId', $this->getGcSource())
                ->addParam('orderId', $transaction_id)
                ->addParam('customerId', $customers_id)
                ->addParam('shippingAddresseFirstName', $strVorname)
                ->addParam('shippingAddresseLastName', $strNachname)
                ->addParam('shippingCompany', $strFirma)
                ->addParam('shippingAdditionalAddressInformation', $strAdditionalAdr)
                ->addParam('shippingStreet', $strStrasse)
                ->addParam('shippingZipCode', $strPLZ)
                ->addParam('shippingCity', $strOrt)
                ->addParam('shippingCountry', $strLand)
                ->addParam('shippingEmail', $strMail)
                ->addParam('cart', $oCart)
                ->addParam('shoppingCartType', $strCartType)
                ->submit();

              if ($request->requestHasSucceeded()) {
                $request->redirectCustomerToPaymentProvider();
                exit();
              }
              else {
                $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage($request->getResponseParam('rc'), $this->lang);
              }
            }
            catch (Exception $e) {
              $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $this->lang) . $e->getMessage();
            }

            break;

          case 'gc_directdebit': //gc_directdebit
            $merchantId = $this->data[$subModule]['merchantId'];
            $projectId = $this->data[$subModule]['projectId'];
            $secure = $this->data[$subModule]['secure'];

            try {

              $request = new GiroCheckout_SDK_Request('directDebitTransactionWithPaymentPage');
              $request->setSecret($secure);
              $request->addParam('merchantId', $merchantId)
                ->addParam('projectId', $projectId)
                ->addParam('merchantTxId', $transaction_id)
                ->addParam('amount', $amount * 100)
                ->addParam('currency', $currency)
                ->addParam('purpose', $vwz)
                ->addParam('urlRedirect', $urlRedirect)
                ->addParam('urlNotify', $urlNotify)
                ->addParam('sourceId', $this->getGcSource())
                ->addParam('orderId', $transaction_id)
                ->addParam('customerId', $customers_id)
                ->submit();

              if ($request->requestHasSucceeded()) {
                $request->redirectCustomerToPaymentProvider();
                exit();
              }
              else {
                $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage($request->getResponseParam('rc'), $this->lang);
              }
            }
            catch (Exception $e) {
              $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $this->lang) . $e->getMessage();
            }
            break;

          case 'gc_creditcard': //gc_creditcard
            $merchantId = $this->data[$subModule]['merchantId'];
            $projectId = $this->data[$subModule]['projectId'];
            $secure = $this->data[$subModule]['secure'];

            try {
              // Adding fields for 3-D Secure 2.0
              $aTdsOptionalInfo = new stdClass();
              $tds2Address = "";
              $tds2Postcode = "";
              $tds2City = "";
              $tds2Country = "";
              $shipAddressAddress = "";
              $shipAddressPostcode = "";
              $shipAddressCity = "";
              $shipAddressCountry = "";
              $strEmail = $order->order_data["customers_email_address"];

              if ($order->order_data) {
                $tds2Address = $order->order_data["delivery_street_address"];
                $tds2Postcode = $order->order_data["delivery_postcode"];
                $tds2City = $order->order_data["delivery_city"];
                $tds2Country = $order->order_data["delivery_country_code"];
              }

              if ($order->order_data) {
                $shipAddressAddress = $order->order_data["delivery_street_address"];
                $shipAddressPostcode = $order->order_data["delivery_postcode"];
                $shipAddressCity = $order->order_data["delivery_city"];
                $shipAddressCountry = $order->order_data["delivery_country_code"];
              }

              $aTdsOptionalInfo->email = $strEmail; // Optional email address

              if ($tds2Address != $shipAddressAddress || $tds2City != $shipAddressCity ||
                $tds2Postcode != $shipAddressPostcode || $tds2Country != $shipAddressCountry
              ) {
                $aTdsOptionalInfo->shippingAddress = new stdClass();
                $aTdsOptionalInfo->shippingAddress->country = $shipAddressCountry;
                $aTdsOptionalInfo->shippingAddress->line1 = $shipAddressAddress;
                $aTdsOptionalInfo->shippingAddress->city = $shipAddressCity;
                $aTdsOptionalInfo->shippingAddress->postcode = $shipAddressPostcode;
              }

              // Shipping address matches billing address, array( "true", "false" );
              if ($tds2Address == $shipAddressAddress && $tds2City == $shipAddressCity &&
                $tds2Postcode == $shipAddressPostcode && $tds2Country == $shipAddressCountry
              ) {
                $aTdsOptionalInfo->addressesMatch = "true";
              }
              else {
                $aTdsOptionalInfo->addressesMatch = "false";
              }

              $request = new GiroCheckout_SDK_Request('creditCardTransaction');
              $request->setSecret($secure);
              $request->addParam('merchantId', $merchantId)
                ->addParam('projectId', $projectId)
                ->addParam('merchantTxId', $transaction_id)
                ->addParam('amount', $amount * 100)
                ->addParam('currency', $currency)
                ->addParam('purpose', $vwz)
                ->addParam('locale', $this->lang)
                ->addParam('urlRedirect', $urlRedirect)
                ->addParam('urlNotify', $urlNotify)
                ->addParam('sourceId', $this->getGcSource())
                ->addParam('orderId', $transaction_id)
                ->addParam('customerId', $customers_id);

              if (!empty($tds2Address) && !empty($tds2Postcode) && !empty($tds2City) && !empty($tds2Country)) {
                $request->addParam('tds2Address', $tds2Address)
                  ->addParam('tds2Postcode', $tds2Postcode)
                  ->addParam('tds2City', $tds2City)
                  ->addParam('tds2Country', $tds2Country);
              }

              $request->addParam('tds2Optional', json_encode($aTdsOptionalInfo));
              $request->submit();

              if ($request->requestHasSucceeded()) {
                $request->redirectCustomerToPaymentProvider();
                exit();
              }
              else {
                $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage($request->getResponseParam('resultPayment'), $this->lang);
              }
            }
            catch (Exception $e) {
              $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $this->lang);
            }

            break;

          case 'gc_eps': //gc_eps
            $merchantId = $this->data[$subModule]['merchantId'];
            $projectId = $this->data[$subModule]['projectId'];
            $secure = $this->data[$subModule]['secure'];

            try {
              //Sends request to Girocheckout.
              $request = new GiroCheckout_SDK_Request('epsTransaction');
              $request->setSecret($secure);
              $request->addParam('merchantId', $merchantId)
                ->addParam('projectId', $projectId)
                ->addParam('merchantTxId', $transaction_id)
                ->addParam('amount', $amount * 100)
                ->addParam('currency', $currency)
                ->addParam('purpose', $vwz)
                ->addParam('urlRedirect', $urlRedirect)
                ->addParam('urlNotify', $urlNotify)
                ->addParam('sourceId', $this->getGcSource())
                ->addParam('orderId', $transaction_id)
                ->addParam('customerId', $customers_id)
                ->submit();

              if ($request->requestHasSucceeded()) {
                $request->redirectCustomerToPaymentProvider();
                exit();
              }
              else {
                $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage($request->getResponseParam('rc'), $this->lang);
              }
            }
            catch (Exception $e) {
              $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $this->lang);
            }
            break;

          case 'gc_ideal': //gc_ideal
            $merchantId = $this->data[$subModule]['merchantId'];
            $projectId = $this->data[$subModule]['projectId'];
            $secure = $this->data[$subModule]['secure'];

            try {
              //Sends request to Girocheckout.
              $request = new GiroCheckout_SDK_Request('idealPayment');
              $request->setSecret($secure);
              $request->addParam('merchantId', $merchantId)
                ->addParam('projectId', $projectId)
                ->addParam('merchantTxId', $transaction_id)
                ->addParam('amount', $amount * 100)
                ->addParam('currency', $currency)
                ->addParam('purpose', $vwz)
                ->addParam('urlRedirect', $urlRedirect)
                ->addParam('urlNotify', $urlNotify)
                ->addParam('sourceId', $this->getGcSource())
                ->addParam('orderId', $transaction_id)
                ->addParam('customerId', $customers_id)
                ->submit();

              if ($request->requestHasSucceeded()) {
                $request->redirectCustomerToPaymentProvider();
                exit();
              }
              else {
                $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage($request->getResponseParam('rc'), $this->lang);
              }
            }
            catch (Exception $e) {
              $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $this->lang);
            }
            break;

          case 'gc_sofortuw':
            $merchantId = $this->data[$subModule]['merchantId'];
            $projectId = $this->data[$subModule]['projectId'];
            $secure = $this->data[$subModule]['secure'];

            try {
              $request = new GiroCheckout_SDK_Request('sofortuwTransaction');
              $request->setSecret($secure);
              $request->addParam('merchantId', $merchantId)
                ->addParam('projectId', $projectId)
                ->addParam('merchantTxId', $transaction_id)
                ->addParam('amount', $amount * 100)
                ->addParam('currency', $currency)
                ->addParam('purpose', $vwz)
                ->addParam('urlRedirect', $urlRedirect)
                ->addParam('urlNotify', $urlNotify)
                ->addParam('sourceId', $this->getGcSource())
                ->addParam('orderId', $transaction_id)
                ->addParam('customerId', $customers_id)
                ->submit();

              if ($request->requestHasSucceeded()) {
                $request->redirectCustomerToPaymentProvider();
                exit();
              }
              else {
                $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage($request->getResponseParam('resultPayment'), $this->lang);
              }
            }
            catch (Exception $e) {
              $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $this->lang);
            }

            break;

          case 'gc_paydirekt':
            $merchantId = $this->data[$subModule]['merchantId'];
            $projectId = $this->data[$subModule]['projectId'];
            $secure = $this->data[$subModule]['secure'];
            $strVorname = $order->order_data["delivery_firstname"];
            $strNachname = $order->order_data["delivery_lastname"];
            $strFirma = $order->order_data["delivery_company"];
            $strAdditionalAdr = $order->order_data["comments"];
            $strStrasse = $order->order_data["delivery_street_address"];
            $strPLZ = $order->order_data["delivery_postcode"];
            $strOrt = $order->order_data["delivery_city"];
            $strLand = $order->order_data["delivery_country_code"];
            $strMail = $order->order_data["customers_email_address"];

            try {
              $oCart = new GiroCheckout_SDK_Request_Cart();

              foreach ($order->order_products as $oProduct) {
                $oCart->addItem($oProduct["products_name"], trim(round($oProduct["products_quantity"], 2)), trim(round($oProduct["products_price"]["plain"], 2)) * 100, $oProduct["products_model"]);
              }

              $request = new GiroCheckout_SDK_Request('paydirektTransaction');
              $request->setSecret($secure);
              $request->addParam('merchantId', $merchantId)
                ->addParam('projectId', $projectId)
                ->addParam('merchantTxId', $transaction_id)
                ->addParam('amount', $amount * 100)
                ->addParam('currency', $currency)
                ->addParam('purpose', $vwz)
                ->addParam('shippingAmount', $amountShipping * 100)
                ->addParam('shippingAddresseFirstName', $strVorname)
                ->addParam('shippingAddresseLastName', $strNachname)
                ->addParam('shippingCompany', $strFirma)
                ->addParam('shippingAdditionalAddressInformation', $strAdditionalAdr)
                ->addParam('shippingStreet', $strStrasse)
                ->addParam('shippingZipCode', $strPLZ)
                ->addParam('shippingCity', $strOrt)
                ->addParam('shippingCountry', $strLand)
                ->addParam('orderAmount', $amountOrder * 100)
                ->addParam('customerMail', $strMail)
                ->addParam('cart', $oCart)
                ->addParam('urlRedirect', $urlRedirect)
                ->addParam('urlNotify', $urlNotify)
                ->addParam('sourceId', $this->getGcSource())
                ->addParam('orderId', $transaction_id)
                ->addParam('customerId', $customers_id)
                ->submit();

              if ($request->requestHasSucceeded()) {
                $request->redirectCustomerToPaymentProvider();
                exit();
              }
              else {
                $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage($request->getResponseParam('resultPayment'), $this->lang);
              }
            }
            catch (Exception $e) {
              $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $this->lang);
            }

            break;

          case 'gc_bluecode':
            $merchantId = $this->data[$subModule]['merchantId'];
            $projectId = $this->data[$subModule]['projectId'];
            $secure = $this->data[$subModule]['secure'];

            try {
              //Sends request to Girocheckout.
              $request = new GiroCheckout_SDK_Request('blueCodeTransaction');
              $request->setSecret($secure);
              $request->addParam('merchantId', $merchantId)
                ->addParam('projectId', $projectId)
                ->addParam('merchantTxId', $transaction_id)
                ->addParam('amount', $amount * 100)
                ->addParam('currency', $currency)
                ->addParam('purpose', $vwz)
                ->addParam('urlRedirect', $urlRedirect)
                ->addParam('urlNotify', $urlNotify)
                ->addParam('sourceId', $this->getGcSource())
                ->addParam('orderId', $transaction_id)
                ->addParam('customerId', $customers_id)
                ->submit();

              if ($request->requestHasSucceeded()) {
                $request->redirectCustomerToPaymentProvider();
                exit();
              }
              else {
                $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage($request->getResponseParam('rc'), $this->lang);
              }
            }
            catch (Exception $e) {
              $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $this->lang);
            }
            break;

          case 'gc_maestro':
            $merchantId = $this->data[$subModule]['merchantId'];
            $projectId = $this->data[$subModule]['projectId'];
            $secure = $this->data[$subModule]['secure'];

            try {
              //Sends request to Girocheckout.
              $request = new GiroCheckout_SDK_Request('maestroTransaction');
              $request->setSecret($secure);
              $request->addParam('merchantId', $merchantId)
                ->addParam('projectId', $projectId)
                ->addParam('merchantTxId', $transaction_id)
                ->addParam('amount', $amount * 100)
                ->addParam('currency', $currency)
                ->addParam('purpose', $vwz)
                ->addParam('urlRedirect', $urlRedirect)
                ->addParam('urlNotify', $urlNotify)
                ->addParam('sourceId', $this->getGcSource())
                ->addParam('orderId', $transaction_id)
                ->addParam('customerId', $customers_id)
                ->submit();

              if ($request->requestHasSucceeded()) {
                $request->redirectCustomerToPaymentProvider();
                exit();
              }
              else {
                $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage($request->getResponseParam('rc'), $this->lang);
              }
            }
            catch (Exception $e) {
              $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $this->lang);
            }
            break;
        }

        if (strlen($strError) > 0) {
          $erMsg = $strError;

          self::setErrorMsg($erMsg);
          $tmp_link = $xtLink->_link(array(
            'page' => 'checkout',
            'paction' => 'payment',
            'params' => $xtLink->_getParams() . '&error=' . $erMsg,
            'conn' => 'SSL'
          ));
        }
      }
    }

    return $tmp_link;
  }

  /**
   *
   * @author S-Public Services
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, S-Public Services
   * @return bool
   */
  function pspSuccess() {
    return true;
  }

  /**
   * Load the plugin payments method texts.
   *
   * @author S-Public Services
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, S-Public Services
   * @return none
   */
  function loadText() {
    $this->aTexts['de']['merchantid'] = "Verkäufer-ID";
    $this->aTexts['en']['merchantid'] = "Merchant ID";
    $this->aTexts['de']['projectid'] = "Projekt-ID";
    $this->aTexts['en']['projectid'] = "Project ID";
    $this->aTexts['de']['passphrase'] = "Projekt-Passwort";
    $this->aTexts['en']['passphrase'] = "Project password";
    $this->aTexts['de']['info'] = "Die Daten für die Einstellungen entnehmen Sie bitte dem Projekt-Profil in Ihrem GiroCockpit (www.girocockpit.de).";
    $this->aTexts['en']['info'] = "";

    $this->aTexts['de']['gc_giropay']['title'] = "giropay - online überweisen";
    $this->aTexts['en']['gc_giropay']['title'] = "giropay - online bank transfer";
    $this->aTexts['de']['gc_giropay']['description'] = "Mit giropay zahlen Sie einfach, schnell und sicher im Online-Banking Ihrer teilnehmenden Bank oder Sparkasse. Sie werden direkt zum Online-Banking Ihrer Bank weitergeleitet, wo Sie die Überweisung durch Eingabe von PIN und TAN freigeben.";
    $this->aTexts['en']['gc_giropay']['description'] = "With giropay you pay simply, fast and secure by the online banking of your participating bank. You will be redirected to the online banking of your bank where you authorize the credit transfer.";
    $this->aTexts['de']['gc_giropay']['bank_selection'] = "Ihre Bank";
    $this->aTexts['en']['gc_giropay']['bank_selection'] = "bank";

    $this->aTexts['de']['gc_directdebit']['title'] = "Lastschrift";
    $this->aTexts['en']['gc_directdebit']['title'] = "direct debit";
    $this->aTexts['de']['gc_directdebit']['description'] = "Hiermit ermächtigen Sie uns, den fälligen Betrag von Ihrem Konto mittels Lastschrift einzuziehen.";
    $this->aTexts['en']['gc_directdebit']['description'] = "Hereby you authorize us to charge your current account with the invoiced amount by direct debit.";

    $this->aTexts['de']['gc_creditcard']['title'] = "Kreditkarte";
    $this->aTexts['en']['gc_creditcard']['title'] = "credit card";
    $this->aTexts['de']['gc_creditcard']['description'] = "Im nächsten Schritt werden Sie aufgefordert, Ihre Kreditkartendaten einzugeben. Wenn Ihre Kreditkarte für 3D-Secure (Verified by Visa bzw. MasterCard SecureCode) freigeschaltet ist, werden Sie während des Bezahlprozesses zur Sicherheitsabfrage Ihrer Bank weitergeleitet.";
    $this->aTexts['en']['gc_creditcard']['description'] = "On the following secure website, you are asked to enter your credit card data. If your credit card is enabled for 3D Secure (Verified by Visa or MasterCard Secure Code), additional information will be passed for a security check at your bank.";

    $this->aTexts['de']['gc_ideal']['title'] = "iDEAL - Online überweisen (NL)";
    $this->aTexts['en']['gc_ideal']['title'] = "iDEAL - online bank transfer (NL)";
    $this->aTexts['de']['gc_ideal']['description'] = "iDEAL ist das Online Überweisungssystem der niederländischen Banken, das auf dem etablierten Onlinebanking-Verfahren der Kreditinstitute basiert. Im folgenden müssen Sie Ihre Bank auswählen und können anschließend die Zahlung in Ihrem Online-Banking durchführen.";
    $this->aTexts['en']['gc_ideal']['description'] = "With iDEAL you pay simply, fast and secure by online banking of your bank. You will be redirected to the online banking of your bank where you authorize the credit transfer with PIN and TAN.";

    $this->aTexts['de']['gc_eps']['title'] = "eps - Online überweisung";
    $this->aTexts['en']['gc_eps']['title'] = "eps - online bank transfer (NL)";
    $this->aTexts['de']['gc_eps']['description'] = "Mit eps Online-Überweisung zahlen Sie einfach, schnell und sicher im Online-Banking Ihrer Bank. Im nächsten Schritt werden Sie direkt zum Online-Banking Ihrer Bank weitergeleitet, wo Sie die Zahlung durch Eingabe von PIN und TAN freigeben.";
    $this->aTexts['en']['gc_eps']['description'] = "With eps online transfer you pay simply, fast and secure by online banking of your bank. Your will be redirected to the online banking at your bank where you authorize the credit transfer with PIN and TAN.";
    $this->aTexts['de']['gc_eps']['bank_selection'] = "Ihre Bank";
    $this->aTexts['en']['gc_eps']['bank_selection'] = "bank";

    $this->aTexts['de']['gc_paydirekt']['title'] = "Paydirekt";
    $this->aTexts['en']['gc_paydirekt']['title'] = "Paydirekt";
    $this->aTexts['de']['gc_paydirekt']['description'] = "Das Online-Bezahlverfahren Ihrer Banken und Sparkassen. Sie werden auf die Webseiten von Paydirekt weiter geleitet, wo Sie sich mit Ihren persönlichen Zugangsdaten anmelden und dann die Zahlung bestätigen. Sie benötigen vorab eine Registrierung bei Paydirekt.";
    $this->aTexts['en']['gc_paydirekt']['description'] = "The online payment method of the German banks and Sparkassen. You will be redirected to the Paydirekt web site, where you will log into your personal account and authorize the payment. You require prior registration with Paydirekt. This payment method only works with German banking accounts.";

    $this->aTexts['de']['gc_sofortuw']['title'] = "SOFORT Überweisung";
    $this->aTexts['en']['gc_sofortuw']['title'] = "SOFORT Überweisung";
    $this->aTexts['de']['gc_sofortuw']['description'] = "Mit SOFORT Überweisung geht das online Bezahlen einfach wie nie! Sie nutzen dabei ganz bequem Ihre vertrauten Online-Banking Daten und sind so auf der sicheren Seite. Sie werden auf die SOFORT-Webseiten weiter geleitet, geben dort Ihre Bankdaten ein und lösen die Online-Überweisung aus.";
    $this->aTexts['en']['gc_sofortuw']['description'] = "Direct payment via online banking. Make payments the easy way with your own online banking details – fast and direct. You will be redirected to the web site of SOFORT, where you will enter your banking data and initiate the electronic wire transfer.";

    $this->aTexts['de']['gc_bluecode']['title'] = "Bluecode";
    $this->aTexts['en']['gc_bluecode']['title'] = "Bluecode";
    $this->aTexts['de']['gc_bluecode']['description'] = "Bluecode funktioniert wie Bargeld – nur auf Ihrem Smartphone. Laden Sie die App jetzt herunter und bezahlen Sie bargeldlos bei über 4500 Partnern. Schnell. Sicher. Einfach.";
    $this->aTexts['en']['gc_bluecode']['description'] = "Bluecode works like cash — just on your smartphone. Download the App now and do cashless payments at over 4500 partners. Fast. Secure. Simple.";

    $this->aTexts['de']['gc_maestro']['title'] = "Maestro";
    $this->aTexts['en']['gc_maestro']['title'] = "Maestro";
    $this->aTexts['de']['gc_maestro']['description'] = "Maestro ist Ihre Bankkarte für den Geldautomaten, mit der Sie jetzt auch online bezahlen können. Es ist eine Debit-Karte, so dass Sie nie mehr ausgeben können, als Sie auf Ihrem Bankkonto haben.";
    $this->aTexts['en']['gc_maestro']['description'] = "Maestro is your ATM banking card that you can now use to pay online. It's a debit facility, so you can never spend more with your Maestro card than you have in your bank account.";
  }

  /**
   * Returns plugin version.
   *
   * @author S-Public Services
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, S-Public Services
   * @return string
   */
  function getVersion() {
    return $this->version;
  }

  /**
   * Returns plugin and shop version.
   *
   * @author S-Public Services
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, S-Public Services
   * @return string
   */
  function getGcSource() {
    return "xt:Commerce " . _SYSTEM_VERSION . ";xt:Commerce Plugin " . $this->getVersion();
  }

  /**
   * Get the payment purpose.
   *
   * @author S-Public Services
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, S-Public Services
   * @return int
   */
  function getPurpose($purpose, $order) {
    $strPurpose = $purpose;

    $iCustomerNr = $order->customer;
    $strLastName = $order->order_data['delivery_lastname'];
    $strFirstName = $order->order_data['delivery_firstname'];
    $strName = $order->order_data['delivery_firstname'] . ", " . $order->order_data['delivery_lastname'];

    $strShopName = _STORE_NAME;
    $strPurpose = str_replace("{ORDERID}", $order->oID, $strPurpose);
    $strPurpose = str_replace("{CUSTOMERID}", $iCustomerNr, $strPurpose);
    $strPurpose = str_replace("{SHOPNAME}", $strShopName, $strPurpose);
    $strPurpose = str_replace("{CUSTOMERNAME}", $strName, $strPurpose);
    $strPurpose = str_replace("{CUSTOMERFIRSTNAME}", $strFirstName, $strPurpose);
    $strPurpose = str_replace("{CUSTOMERLASTNAME}", $strLastName, $strPurpose);

    return utf8_encode(trim(substr($strPurpose, 0, 27)));
  }

  /**
   * Get the extended logo for creditcards.
   *
   * @author S-Public Services
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, S-Public Services
   * @return string
   */
  function getExtendedLogo() {
    $subModule = 'gc_creditcard';
    $visa_msc = ($this->data[$subModule]['visamaster'] == 'true') ? 1 : 0;
    $amex = ($this->data[$subModule]['amex'] == 'true') ? 1 : 0;
    $jcb = ($this->data[$subModule]['jcb'] == 'true') ? 1 : 0;

    if (!$visa_msc && !$amex && !$jcb) {
      $visa_msc = true;
    }

    return GiroCheckout_SDK_Tools::getCreditCardLogoName($visa_msc, $amex, $jcb);
  }

  /**
   * Get the payment purpose for giropay payment.
   *
   * @author S-Public Services GmbH
   * @package GiroCheckout
   * @copyright Copyright (c) 2022, S-Public Services GmbH
   * @return string The purpose format
   */
  function getGiropayPurpose($purpose, $order) {
    $strPurpose = $purpose;

    $iCustomerNr = $order->customer;
    $strLastName = $order->order_data['delivery_lastname'];
    $strFirstName = $order->order_data['delivery_firstname'];
    $strName = $order->order_data['delivery_firstname'] . ", " . $order->order_data['delivery_lastname'];

    $strShopName = _STORE_NAME;
    $strPurpose = str_replace("{ORDERID}", $order->oID, $strPurpose);
    $strPurpose = str_replace("{CUSTOMERID}", $iCustomerNr, $strPurpose);
    $strPurpose = str_replace("{SHOPNAME}", $strShopName, $strPurpose);
    $strPurpose = str_replace("{CUSTOMERNAME}", $strName, $strPurpose);
    $strPurpose = str_replace("{CUSTOMERFIRSTNAME}", $strFirstName, $strPurpose);
    $strPurpose = str_replace("{CUSTOMERLASTNAME}", $strLastName, $strPurpose);

    $bValidPurpose = self::validatePurpose($strPurpose);

    if (!$bValidPurpose) {
      $strPurposeFormat = preg_replace("([^[A-Za-z0-9\+\?\-\:\(\)\.\,\'\/\s])", '', $strPurpose);
    }
    else {
      $strPurposeFormat = $strPurpose;
    }

    $strPurposeFormat = utf8_encode(trim(substr($strPurposeFormat, 0, 27)));

    return $strPurposeFormat;
  }

  /**
   * Validate the payment purpose for giropay payment.
   *
   * @author S-Public Services GmbH
   * @package GiroCheckout
   * @copyright Copyright (c) 2022, S-Public Services GmbH
   * @return array
   */
  function validatePurpose($purpose) {
    $strPurpose = $purpose;

    if (!empty($strPurpose)) {
      if (preg_match("/^[A-Za-z0-9\+\?\-\:\(\)\.\,\'\/\s]+$/", $strPurpose)) {
        return true;
      }
      else {
        return false;
      }
    }

    return true;
  }
}